@@ -3,7 +3,7 @@
  * Author: Rich Gade
  * Project: GGZ Core Client Lib
  * Date: 02/19/01
- * $Id: ggz-config.c 7712 2006-01-02 16:45:48Z josef $
+ * $Id: ggz-config.c 8678 2006-11-27 09:41:30Z josef $
  *
  * Configuration query and module install program.
  *
@@ -598,10 +598,10 @@ static int noregister_all()
 		sprintf(modfile, "%s/%s", copydir, e->d_name);
 		if(load_modfile()) {
 			if(install_mod) {
-				printf("- register %s\n", e->d_name);
+				printf(_("- register %s\n"), e->d_name);
 				ret = install_module();
 			} else if(remove_mod) {
-				printf("- unregister %s\n", e->d_name);
+				printf(_("- unregister %s\n"), e->d_name);
 				ret = remove_module();
 			} else ret = -1;
 		} else {
@@ -644,7 +644,7 @@ static int check_module_file(void)
 {
 	int	global;
 	int	rc;
-	int	e_count, s_count, k_count, g_count;
+	int	e_count, s_count, k_count, g_count, n_count;
 	char	**e_list, **s_list, **k_list, **g_list;
 	char	*str, *str2;
 	int	kill, ok, alt;
@@ -880,6 +880,7 @@ phase_two:
 			continue;
 		}
 		ggz_conf_read_list(global, "Games", k_list[i], &g_count, &g_list);
+		n_count = g_count;
 
 		for(k = 0; k < g_count; k++) {
 			str = ggz_conf_read_string(global, g_list[k], "ProtocolEngine", NULL);
@@ -889,18 +890,18 @@ phase_two:
 				printf(_("ERR Section %s doesn't exist in %s, removed reference\n"),
 					g_list[k], k_list[i]);
 				ggz_free(g_list[k]);
-				g_count -= 1;
-				if(g_count) {
-					if(k <= g_count)
-						g_list[k] = g_list[g_count];
-					ggz_conf_write_list(global, "Games", k_list[i], g_count, g_list);
+				n_count -= 1;
+				if(n_count) {
+					if(k <= n_count)
+						g_list[k] = g_list[n_count];
+					ggz_conf_write_list(global, "Games", k_list[i], n_count, g_list);
 				} else {
 					ggz_conf_remove_key(global, "Games", k_list[i]);
 				}
 			}
 			else ggz_free(str);
 		}
-		for(k = 0; k < g_count; k++)
+		for(k = 0; k < n_count; k++)
 			ggz_free(g_list[k]);
 		ggz_free(g_list);
 	}
@@ -1024,7 +1025,7 @@ int main(int argc, char *argv[])
 			case 'u':
 				printf(_("Usage:\n"));
 				printf("\tggz-config --install --modfile=<module.dsc> [--force]\n");
-				printf("\tggz-config --remove --modfile=<module.dsc> [--force]\n");
+				printf("\tggz-config --remove --modfile=<module.dsc>\n");
 				printf("\tggz-config --check\n");
 				return 0;
 				break;
@@ -1057,7 +1058,7 @@ int main(int argc, char *argv[])
 				return 0;
 				break;
 			case 'g':
-				printf("%s\n", GAMEDIR);
+				printf("%s\n", GGZEXECMODDIR);
 				return 0;
 				break;
 			case 'd':
