@@ -3,7 +3,7 @@
  * Author: Brent Hendricks
  * Project: GGZ Core Client Lib
  * Date: 9/15/00
- * $Id: init.c 7889 2006-03-07 09:57:32Z josef $
+ * $Id: init.c 8907 2007-01-14 14:37:42Z oojah $
  *
  * Initialization code
  *
@@ -41,9 +41,12 @@
 
 #include <locale.h>
 #include <libintl.h>
+#include <signal.h>
 
 int ggzcore_init(GGZOptions options)
 {
+	int ret = 0;
+
 #if 0
 	if (options.flags & GGZ_OPT_PARSER) {
 		ggz_debug(GGZCORE_DBG_CONF, "Parsing global conf file: %s",
@@ -55,11 +58,12 @@ int ggzcore_init(GGZOptions options)
 	}
 #endif
 
+	/* This catalog must be preloaded by applications */
 	bindtextdomain("ggzcore", PREFIX "/share/locale");
 
 	/* Initialize various systems */
 	if (options.flags & GGZ_OPT_MODULES)
-		_ggzcore_module_setup();
+		ret = _ggzcore_module_setup();
 
 	if (options.flags & GGZ_OPT_EMBEDDED)
 		_ggzcore_module_set_embedded();
@@ -70,7 +74,12 @@ int ggzcore_init(GGZOptions options)
 	if (options.flags & GGZ_OPT_THREADED_IO)
 		_ggzcore_server_set_threaded_io();
 
-	return 0;
+#ifndef __MINGW32__
+	/* Do not die if child process dies while we're communicating with it */
+	signal(SIGPIPE, SIG_IGN);
+#endif
+
+	return ret;
 }
 
 
