@@ -3,7 +3,7 @@
  * Author: Brent Hendricks
  * Project: GGZ Core Client Lib
  * Date: 9/22/00
- * $Id: netxml.c 7909 2006-03-14 13:50:27Z josef $
+ * $Id: netxml.c 8888 2007-01-10 02:28:31Z jdorje $
  *
  * Code for parsing XML streamed from the server
  *
@@ -60,6 +60,8 @@
 /* For convenience */
 #define XML_BUFFSIZE 8192
 
+#define ARRAY_SIZE(arr) (sizeof(arr) / sizeof((arr)[0]))
+
 #define ATTR ggz_xmlelement_get_attr
 
 
@@ -162,8 +164,8 @@ static void _ggzcore_net_handle_room(GGZNet *, GGZXMLElement *);
 static void _ggzcore_net_handle_player(GGZNet *, GGZXMLElement *);
 static void _ggzcore_net_handle_table(GGZNet *, GGZXMLElement *);
 static void _ggzcore_net_handle_seat(GGZNet *, GGZXMLElement *);
-static void _ggzcore_net_handle_spectator_seat(GGZNet * net,
-					       GGZXMLElement * seat);
+static void _ggzcore_net_handle_spectator(GGZNet * net,
+					  GGZXMLElement * seat);
 static void _ggzcore_net_handle_chat(GGZNet *, GGZXMLElement *);
 static void _ggzcore_net_handle_info(GGZNet *, GGZXMLElement *);
 static void _ggzcore_net_handle_playerinfo(GGZNet *, GGZXMLElement *);
@@ -367,86 +369,6 @@ void _ggzcore_net_disconnect(GGZNet * net)
 }
 
 
-/* Helper function, might go into libggz*/
-char *_ggz_xml_cdata_escape(const char *str)
-{
-	char *new, *q;
-	const char *p;
-	size_t len = 0;
-
-	if(str == NULL)
-		return NULL;
-
-	len = strlen(str);
-
-	for(p = str; *p != '\0'; p++) {
-		if((*p == ']') && (*(p + 1) == ']') && (*(p + 2) == '>')) {
-			len += 3;
-		}
-	}
-
-	if(len == strlen(str))
-		return ggz_strdup(str);
-
-	q = new = ggz_malloc(len + 1);
-	for(p = str; *p != '\0'; p++) {
-		if((*p == ']') && (*(p + 1) == ']') && (*(p + 2) == '>')) {
-			memcpy(q, "]]&gt;", 6);
-			q += 6;
-			p += 2;
-		} else {
-			*q = *p;
-			q++;
-		}
-	}
-	*q = '\0';
-
-	return new;
-}
-
-
-/* Helper function, might go into libggz*/
-char *_ggz_xml_cdata_unescape(const char *str)
-{
-	char *new, *q;
-	const char *p;
-	size_t len = 0;
-
-	if(str == NULL)
-		return NULL;
-
-	len = strlen(str);
-
-	for(p = str; *p != '\0'; p++) {
-		if(!strncmp(p, "]]&gt;", 6)) {
-			p += 5;
-			len += 2;
-		}
-		len++;
-	}
-
-	if(len == strlen(str))
-		return ggz_strdup(str);
-
-	q = new = ggz_malloc(len + 1);
-	for(p = str; *p != '\0'; p++) {
-		if(!strncmp(p, "]]&gt;", 6)) {
-			*q++ = *p++;
-			*q++ = *p++;
-			*q = '>';
-			q++;
-			p += 3;
-		} else {
-			*q = *p;
-			q++;
-		}
-	}
-	*q = '\0';
-
-	return new;
-}
-
-
 /* ggzcore_net_send_XXX() functions for sending messages to the server */
 
 /* Sends login packet.  Login type is an enumerated value.  Password is needed
@@ -473,19 +395,22 @@ int _ggzcore_net_send_login(GGZNet * net, GGZLoginType login_type,
 		break;
 	}
 
-	handle_quoted = _ggz_xml_cdata_escape(handle);
-	password_quoted = _ggz_xml_cdata_escape(password);
-	email_quoted = _ggz_xml_cdata_escape(email);
+	handle_quoted = ggz_xml_escape(handle);
+	password_quoted = ggz_xml_escape(password);
+	email_quoted = ggz_xml_escape(email);
 
-	_ggzcore_net_send_line(net, "<LANGUAGE>%s</LANGUAGE>", language);
+	if (language) {
+		_ggzcore_net_send_line(net, "<LANGUAGE>%s</LANGUAGE>",
+				       language);
+	}
 	_ggzcore_net_send_line(net, "<LOGIN TYPE='%s'>", type);
-	_ggzcore_net_send_line(net, "<NAME><![CDATA[%s]]></NAME>", handle_quoted);
+	_ggzcore_net_send_line(net, "<NAME>%s</NAME>", handle_quoted);
 
 	if ((login_type == GGZ_LOGIN || (login_type == GGZ_LOGIN_NEW)) && password)
-		_ggzcore_net_send_line(net, "<PASSWORD><![CDATA[%s]]></PASSWORD>",
+		_ggzcore_net_send_line(net, "<PASSWORD>%s</PASSWORD>",
 				       password_quoted);
 	if (login_type == GGZ_LOGIN_NEW && email)
-		_ggzcore_net_send_line(net, "<EMAIL><![CDATA[%s]]></EMAIL>",
+		_ggzcore_net_send_line(net, "<EMAIL>%s</EMAIL>",
 				       email_quoted);
 
 	status = _ggzcore_net_send_line(net, "</LOGIN>");
@@ -627,14 +552,14 @@ int _ggzcore_net_send_chat(GGZNet * net, const GGZChatType type,
 		chat_text = msg;
 	}
 
-	chat_text_quoted = _ggz_xml_cdata_escape(chat_text);
+	chat_text_quoted = ggz_xml_escape(chat_text);
 
 	switch (type) {
 	case GGZ_CHAT_NORMAL:
 	case GGZ_CHAT_ANNOUNCE:
 	case GGZ_CHAT_TABLE:
 		result = _ggzcore_net_send_line(net,
-						"<CHAT TYPE='%s'><![CDATA[%s]]></CHAT>",
+						"<CHAT TYPE='%s'>%s</CHAT>",
 						type_str, chat_text_quoted);
 		break;
 	case GGZ_CHAT_BEEP:
@@ -644,12 +569,11 @@ int _ggzcore_net_send_chat(GGZNet * net, const GGZChatType type,
 		break;
 	case GGZ_CHAT_PERSONAL:
 		result = _ggzcore_net_send_line(net,
-						"<CHAT TYPE='%s' TO='%s'><![CDATA[%s]]></CHAT>",
+						"<CHAT TYPE='%s' TO='%s'>%s</CHAT>",
 						type_str, player,
 						chat_text_quoted);
 		break;
 	case GGZ_CHAT_UNKNOWN:
-	default:
 		/* Returning an error would mean a *network* error,
 		   which isn't the case. */
 		result = 0;
@@ -671,6 +595,83 @@ int _ggzcore_net_send_chat(GGZNet * net, const GGZChatType type,
 }
 
 
+/* Send an <ADMIN> tag for gag/ungag/kick/... */
+int _ggzcore_net_send_room_admin(GGZNet * net, const GGZAdminType type,
+				 const char *player, const char *reason)
+{
+	const char *reason_text;
+	char *reason_text_quoted;
+	char *my_text = NULL;
+	int result = -1;
+
+	ggz_debug(GGZCORE_DBG_NET, "Sending administrative action");
+
+	/* Truncate a message that is too long. See _ggzcore_net_send_chat. */
+	if (reason && strlen(reason) > net->chat_size) {
+		ggz_error_msg("Truncating too-long reason message.");
+		my_text = ggz_malloc(net->chat_size + 1);
+		strncpy(my_text, reason, net->chat_size);
+		my_text[net->chat_size] = '\0';
+		reason_text = my_text;
+	} else {
+		reason_text = reason;
+	}
+
+	reason_text_quoted = ggz_xml_escape(reason_text);
+	/* FIXME: player string must be quoted also (same for chat send) */
+
+	switch (type) {
+	case GGZ_ADMIN_GAG:
+		result = _ggzcore_net_send_line(net,
+						"<ADMIN ACTION='gag' PLAYER='%s'/>",
+						player);
+		break;
+	case GGZ_ADMIN_UNGAG:
+		result = _ggzcore_net_send_line(net,
+						"<ADMIN ACTION='ungag' PLAYER='%s'/>",
+						player);
+		break;
+	case GGZ_ADMIN_KICK:
+		result = _ggzcore_net_send_line(net,
+						"<ADMIN ACTION='kick' PLAYER='%s'>",
+						player);
+		result |= _ggzcore_net_send_line(net,
+						"<REASON>%s</REASON>",
+						reason_text_quoted);
+		result |= _ggzcore_net_send_line(net,
+						"</ADMIN>");
+		break;
+	case GGZ_ADMIN_BAN:
+	case GGZ_ADMIN_UNKNOWN:
+		/* Not yet in use. */
+		break;
+	}
+
+	if (reason_text_quoted)
+		ggz_free(reason_text_quoted);
+
+	if (my_text) {
+		ggz_free(my_text);
+	}
+
+	return result;
+}
+
+
+int _ggzcore_net_send_perm_admin(GGZNet * net, GGZPlayer *player,
+				 GGZPerm perm, bool set)
+{
+	const char *permname = ggz_perm_to_string(perm);
+	const char *setname = bool_to_str(set);
+	const char *playername = _ggzcore_player_get_name(player);
+
+	return _ggzcore_net_send_line(net,
+				      "<ADMINPERM PLAYER='%s' PERM='%s' "
+				      "VALUE='%s'/>",
+				      playername, permname, setname);
+}
+
+
 int _ggzcore_net_send_player_info(GGZNet * net, int seat_num)
 {
 	int result;
@@ -730,18 +731,28 @@ int _ggzcore_net_send_table_launch(GGZNet * net, GGZTable * table)
 static int _ggzcore_net_send_table_seat(GGZNet * net, GGZTableSeat * seat)
 {
 	const char *type;
+	int ret;
 
 	ggz_debug(GGZCORE_DBG_NET, "Sending seat info");
 
 	type = ggz_seattype_to_string(seat->type);
 
-	if (!seat->name)
-		return _ggzcore_net_send_line(net,
-					      "<SEAT NUM='%d' TYPE='%s'/>",
-					      seat->index, type);
-	return _ggzcore_net_send_line(net,
-				      "<SEAT NUM='%d' TYPE='%s'>%s</SEAT>",
-				      seat->index, type, seat->name);
+
+	if (!seat->name) {
+		ret = _ggzcore_net_send_line(net,
+					     "<SEAT NUM='%d' TYPE='%s'/>",
+					     seat->index, type);
+	} else {
+		const char *name_quoted = ggz_xml_escape(seat->name);
+
+		ret = _ggzcore_net_send_line(net,
+					     "<SEAT NUM='%d' TYPE='%s'>%s"
+					     "</SEAT>",
+					     seat->index, type, name_quoted);
+		ggz_free(name_quoted);
+	}
+
+	return ret;
 }
 
 
@@ -802,7 +813,7 @@ int _ggzcore_net_send_table_seat_update(GGZNet * net, GGZTable * table,
 					GGZTableSeat * seat)
 {
 	const GGZRoom *room = ggzcore_table_get_room(table);
-	int room_id = ggzcore_room_get_id(room);
+	int room_id = _ggzcore_room_get_id(room);
 	int id = ggzcore_table_get_id(table);
 	int num_seats = ggzcore_table_get_num_seats(table);
 
@@ -823,7 +834,7 @@ int _ggzcore_net_send_table_desc_update(GGZNet * net, GGZTable * table,
 					const char *desc)
 {
 	const GGZRoom *room = ggzcore_table_get_room(table);
-	int room_id = ggzcore_room_get_id(room);
+	int room_id = _ggzcore_room_get_id(room);
 	int id = ggzcore_table_get_id(table);
 	char *desc_quoted;
 
@@ -849,7 +860,7 @@ int _ggzcore_net_send_table_boot_update(GGZNet * net, GGZTable * table,
 					GGZTableSeat * seat)
 {
 	const GGZRoom *room = ggzcore_table_get_room(table);
-	int room_id = ggzcore_room_get_id(room);
+	int room_id = _ggzcore_room_get_id(room);
 	int id = ggzcore_table_get_id(table);
 
 	ggz_debug(GGZCORE_DBG_NET, "Sending boot of player %s.",
@@ -1035,63 +1046,50 @@ static void _ggzcore_net_dump_data(GGZNet * net, char *data, int size)
 static GGZXMLElement *_ggzcore_net_new_element(const char *tag,
 					       const char *const *attrs)
 {
-	void (*process_func) ();
+	void (*process_func) (GGZNet *net, GGZXMLElement *element) = NULL;
+	int i;
+	struct {
+		const char *tag;
+		void (*process_func)(GGZNet *net, GGZXMLElement *element);
+	} tags[] = {
+#define TAG(t) {#t, _ggzcore_net_handle_ ## t}
+	  TAG(server),
+	  TAG(options),
+	  TAG(motd),
+	  TAG(result),
+	  TAG(list),
+	  TAG(update),
+	  TAG(game),
+	  TAG(protocol),
+	  TAG(allow),
+	  TAG(about),
+	  TAG(bot),
+	  TAG(room),
+	  TAG(player),
+	  TAG(table),
+	  TAG(seat),
+	  TAG(spectator),
+	  TAG(leave),
+	  TAG(join),
+	  TAG(chat),
+	  TAG(info),
+	  TAG(playerinfo),
+	  TAG(desc),
+	  TAG(password),
+	  TAG(ping),
+	  TAG(session)
+#undef TAG
+	};
+	typedef void (*ggzxmlfunc)(void *, GGZXMLElement *);
 
-	/* FIXME: Could we do this with a table lookup? */
-	if (strcasecmp(tag, "SERVER") == 0)
-		process_func = _ggzcore_net_handle_server;
-	else if (strcasecmp(tag, "OPTIONS") == 0)
-		process_func = _ggzcore_net_handle_options;
-	else if (strcasecmp(tag, "MOTD") == 0)
-		process_func = _ggzcore_net_handle_motd;
-	else if (strcasecmp(tag, "RESULT") == 0)
-		process_func = _ggzcore_net_handle_result;
-	else if (strcasecmp(tag, "LIST") == 0)
-		process_func = _ggzcore_net_handle_list;
-	else if (strcasecmp(tag, "UPDATE") == 0)
-		process_func = _ggzcore_net_handle_update;
-	else if (strcasecmp(tag, "GAME") == 0)
-		process_func = _ggzcore_net_handle_game;
-	else if (strcasecmp(tag, "PROTOCOL") == 0)
-		process_func = _ggzcore_net_handle_protocol;
-	else if (strcasecmp(tag, "ALLOW") == 0)
-		process_func = _ggzcore_net_handle_allow;
-	else if (strcasecmp(tag, "ABOUT") == 0)
-		process_func = _ggzcore_net_handle_about;
-	else if (strcasecmp(tag, "BOT") == 0)
-		process_func = _ggzcore_net_handle_bot;
-	else if (strcasecmp(tag, "ROOM") == 0)
-		process_func = _ggzcore_net_handle_room;
-	else if (strcasecmp(tag, "PLAYER") == 0)
-		process_func = _ggzcore_net_handle_player;
-	else if (strcasecmp(tag, "TABLE") == 0)
-		process_func = _ggzcore_net_handle_table;
-	else if (strcasecmp(tag, "SEAT") == 0)
-		process_func = _ggzcore_net_handle_seat;
-	else if (strcasecmp(tag, "SPECTATOR") == 0)
-		process_func = _ggzcore_net_handle_spectator_seat;
-	else if (strcasecmp(tag, "LEAVE") == 0)
-		process_func = _ggzcore_net_handle_leave;
-	else if (strcasecmp(tag, "JOIN") == 0)
-		process_func = _ggzcore_net_handle_join;
-	else if (strcasecmp(tag, "CHAT") == 0)
-		process_func = _ggzcore_net_handle_chat;
-	else if (strcasecmp(tag, "INFO") == 0)
-		process_func = _ggzcore_net_handle_info;
-	else if (strcasecmp(tag, "PLAYERINFO") == 0)
-		process_func = _ggzcore_net_handle_playerinfo;
-	else if (strcasecmp(tag, "DESC") == 0)
-		process_func = _ggzcore_net_handle_desc;
-	else if (strcasecmp(tag, "PASSWORD") == 0)
-		process_func = _ggzcore_net_handle_password;
-	else if (strcasecmp(tag, "PING") == 0)
-		process_func = _ggzcore_net_handle_ping;
-	else if (strcasecmp(tag, "SESSION") == 0)
-		process_func = _ggzcore_net_handle_session;
-	else
-		process_func = NULL;
+	for (i = 0; i < ARRAY_SIZE(tags); i++) {
+		if (strcasecmp(tags[i].tag, tag) == 0) {
+			process_func = tags[i].process_func;
+			break;
+		}
+	}
 
-	return ggz_xmlelement_new(tag, attrs, process_func, NULL);
+	return ggz_xmlelement_new(tag, attrs, (ggzxmlfunc)process_func, NULL);
 }
 
 
@@ -1230,7 +1228,7 @@ static void _ggzcore_net_handle_result(GGZNet * net,
 		_ggzcore_room_set_table_leave_status(room, code);
 	else if (strcasecmp(action, "chat") == 0) {
 		if (code != E_OK) {
-		      GGZErrorEventData error = { status:code };
+			GGZErrorEventData error = { .status = code };
 
 			switch (code) {
 			case E_NOT_IN_ROOM:
@@ -1272,6 +1270,16 @@ static void _ggzcore_net_handle_result(GGZNet * net,
 			_ggzcore_server_event(net->server, GGZ_CHAT_FAIL,
 					      &error);
 		}
+	}
+	else if (strcasecmp(action, "admin") == 0) {
+		if (code != E_OK) {
+			GGZErrorEventData error = { .status = code };
+			snprintf(error.message,
+				 sizeof(error.message),
+				 "Admin action error");
+			_ggzcore_server_event(net->server, GGZ_CHAT_FAIL,
+					      &error);
+		}
 	} else if (strcasecmp(action, "protocol") == 0) {
 		/* These are always errors */
 		switch (code) {
@@ -1454,17 +1462,38 @@ static void _ggzcore_net_room_update(GGZNet * net, GGZXMLElement * update,
 	roomdata = ggz_xmlelement_get_data(update);
 	if (!roomdata)
 		return;
-	id = ggzcore_room_get_id(roomdata);
+	id = _ggzcore_room_get_id(roomdata);
 	room = _ggzcore_server_get_room_by_id(net->server, id);
 
 	if (room) {
 		if (strcasecmp(action, "players") == 0) {
 			players = ggzcore_room_get_num_players(roomdata);
 			_ggzcore_room_set_players(room, players);
+		} else if(strcasecmp(action, "delete") == 0) {
+			/* FIXME: no such function yet for removals */
+			_ggzcore_server_delete_room(net->server, room);
+			_ggzcore_server_event(net->server,
+				GGZ_SERVER_ROOMS_CHANGED,
+				NULL);
+		} else if(strcasecmp(action, "close") == 0) {
+			/* FIXME: mark room as closed? */
+			_ggzcore_room_close(room);
+			_ggzcore_server_event(net->server,
+				GGZ_SERVER_ROOMS_CHANGED,
+				NULL);
+		}
+
+		_ggzcore_room_free(roomdata);
+	} else {
+		if(strcasecmp(action, "add") == 0) {
+			/* FIXME: resize room list (array) first */
+			_ggzcore_server_grow_roomlist(net->server);
+			_ggzcore_server_add_room(net->server, roomdata);
+			_ggzcore_server_event(net->server,
+				GGZ_SERVER_ROOMS_CHANGED,
+				NULL);
 		}
 	}
-
-	_ggzcore_room_free(roomdata);
 }
 
 
@@ -1495,17 +1524,29 @@ static void _ggzcore_net_player_update(GGZNet * net,
 
 	if (strcasecmp(action, "add") == 0) {
 		int from_room = str_to_int(ATTR(update, "FROMROOM"), -2);
+		GGZRoom *from_room_ptr
+		  = _ggzcore_server_get_room_by_id(net->server, from_room);
 
-		_ggzcore_room_add_player(room, player, from_room);
+		_ggzcore_room_add_player(room, player,
+					 (from_room != -2), from_room_ptr);
 	} else if (strcasecmp(action, "delete") == 0) {
 		int to_room = str_to_int(ATTR(update, "TOROOM"), -2);
+		GGZRoom *to_room_ptr
+		  = _ggzcore_server_get_room_by_id(net->server, to_room);
 
-		_ggzcore_room_remove_player(room, player_name, to_room);
+		_ggzcore_room_remove_player(room, player_name,
+					    (to_room != -2), to_room_ptr);
 	} else if (strcasecmp(action, "lag") == 0) {
 		/* FIXME: Should be a player "class-based" event */
 		int lag = ggzcore_player_get_lag(player);
 
 		_ggzcore_room_set_player_lag(room, player_name, lag);
+	} else if (strcasecmp(action, "perms") == 0) {
+		GGZPermset permset = _ggzcore_player_get_perms(player);
+		GGZPlayerType type = _ggzcore_player_get_type(player);
+
+		_ggzcore_room_set_player_perms(room, player_name,
+					       permset, type);
 	} else if (strcasecmp(action, "stats") == 0) {
 		/* FIXME: Should be a player "class-based" event */
 		_ggzcore_room_set_player_stats(room, player);
@@ -1530,7 +1571,7 @@ static void _ggzcore_net_table_update(GGZNet * net, GGZXMLElement * update,
 		/* Assume we're talking about the current room.  This is
 		   no doubt preferable to simply dropping the connection. */
 		room = ggzcore_server_get_cur_room(net->server);
-		room_num = ggzcore_room_get_id(room);
+		room_num = _ggzcore_room_get_id(room);
 	} else
 		room_num = str_to_int(room_str, -1);
 
@@ -2059,6 +2100,7 @@ static void _ggzcore_net_handle_player(GGZNet * net,
 	GGZRoom *room;
 	const char *name, *str_type;
 	int table, lag;
+	GGZPermset perms;
 	GGZXMLElement *parent;
 	const char *parent_tag, *parent_type;
 	int wins, losses, ties, forfeits, rating, ranking, highscore;
@@ -2073,13 +2115,14 @@ static void _ggzcore_net_handle_player(GGZNet * net,
 	name = ATTR(element, "ID");
 	table = str_to_int(ATTR(element, "TABLE"), -1);
 	lag = str_to_int(ATTR(element, "LAG"), 0);
+	perms = str_to_int(ATTR(element, "PERMS"), 0);
 
 	/* Set player's type */
 	type = ggz_string_to_playertype(str_type);
 
 	/* Set up GGZPlayer object */
 	ggz_player = _ggzcore_player_new();
-	_ggzcore_player_init(ggz_player, name, room, table, type, lag);
+	_ggzcore_player_init(ggz_player, name, room, table, type, perms, lag);
 
 	/* FIXME: should these be initialized through an accessor function? */
 	wins = str_to_int(ATTR(element, "WINS"), NO_RECORD);
@@ -2291,8 +2334,8 @@ static void _ggzcore_net_handle_seat(GGZNet * net, GGZXMLElement * element)
 }
 
 /* Functions for <SPECTATOR> tag */
-static void _ggzcore_net_handle_spectator_seat(GGZNet * net,
-					       GGZXMLElement * element)
+static void _ggzcore_net_handle_spectator(GGZNet * net,
+					  GGZXMLElement * element)
 {
 	GGZTableSeat seat_obj;
 	GGZXMLElement *parent;
@@ -2379,7 +2422,6 @@ static void _ggzcore_net_handle_join(GGZNet * net, GGZXMLElement * element)
 static void _ggzcore_net_handle_chat(GGZNet * net, GGZXMLElement * element)
 {
 	const char *msg, *type_str, *from;
-	char *msg_unquoted;
 	GGZRoom *room;
 	GGZChatType type;
 
@@ -2407,13 +2449,8 @@ static void _ggzcore_net_handle_chat(GGZNet * net, GGZXMLElement * element)
 		return;
 	}
 
-	msg_unquoted = _ggz_xml_cdata_unescape(msg);
-
 	room = ggzcore_server_get_cur_room(net->server);
-	_ggzcore_room_add_chat(room, type, from, msg_unquoted);
-
-	if (msg_unquoted)
-		ggz_free(msg_unquoted);
+	_ggzcore_room_add_chat(room, type, from, msg);
 }
 
 
@@ -2526,8 +2563,9 @@ static int str_to_int(const char *str, int dflt)
 {
 	int val;
 
-	if (!str || sscanf(str, "%d", &val) < 1)
-		return dflt;
+	if (!str) return dflt;
+	if (sscanf(str, "0x%x", &val) == 1) return val;
+	if (sscanf(str, "%d", &val) == 1) return val;
 
-	return val;
+	return dflt;
 }
