@@ -3,7 +3,7 @@
  * Author: Brent Hendricks
  * Project: GGZ Core Client Lib
  * Date: 6/5/00
- * $Id: player.c 7889 2006-03-07 09:57:32Z josef $
+ * $Id: player.c 8763 2006-12-27 10:02:33Z jdorje $
  *
  * This fils contains functions for handling players
  *
@@ -54,6 +54,9 @@ struct _GGZPlayer {
 	/* Server ID of table player is at */
 	int table;
 
+	/* Player permissions */
+	GGZPermset perms;
+
 	/* Lag of the player */
 	int lag;
 
@@ -103,6 +106,19 @@ GGZTable *ggzcore_player_get_table(const GGZPlayer * player)
 	return _ggzcore_player_get_table(player);
 }
 
+bool ggzcore_player_has_perm(const GGZPlayer *player, GGZPerm perm)
+{
+	if (!player)
+		return false;
+	return _ggzcore_player_has_perm(player, perm);
+}
+
+int ggzcore_player_set_perm(GGZPlayer *player, GGZPerm perm, bool set)
+{
+	if (!player || perm < 0 || perm >= GGZ_PERM_COUNT || set != !!set)
+		return -1;
+	return _ggzcore_player_set_perm(player, perm, set);
+}
 
 int ggzcore_player_get_lag(const GGZPlayer * player)
 {
@@ -159,6 +175,8 @@ GGZPlayer *_ggzcore_player_new(void)
 	/* Set to invalid table */
 	player->table = -1;
 
+	player->perms = 0;
+
 	/* Assume no lag */
 	player->lag = -1;
 
@@ -178,12 +196,15 @@ void _ggzcore_player_init(GGZPlayer * player,
 			  const char *name,
 			  GGZRoom * room,
 			  const int table,
-			  const GGZPlayerType type, const int lag)
+			  const GGZPlayerType type,
+			  const GGZPermset perms,
+			  const int lag)
 {
 	player->name = ggz_strdup(name);
 	player->room = room;
 	player->table = table;
 	player->type = type;
+	player->perms = perms;
 	player->lag = lag;
 }
 
@@ -251,6 +272,45 @@ GGZTable *_ggzcore_player_get_table(const GGZPlayer * player)
 }
 
 
+GGZPermset _ggzcore_player_get_perms(const GGZPlayer * player)
+{
+	return player->perms;
+}
+
+
+bool _ggzcore_player_has_perm(const GGZPlayer *player, GGZPerm perm)
+{
+	return ggz_perms_is_set(player->perms, perm);
+}
+
+
+/* Not to be confused with _ggzcore_player_set_perm */
+int _ggzcore_player_set_perm(GGZPlayer *player, GGZPerm perm, bool set)
+{
+	GGZServer *server = ggzcore_room_get_server(player->room);
+	GGZNet *net = _ggzcore_server_get_net(server);
+	const char *handle = ggzcore_server_get_handle(server);
+	const GGZPlayer *me = ggzcore_server_get_player(server, handle);
+
+	if (me->type != GGZ_PLAYER_ADMIN) {
+		return -1;
+	}
+
+	if (_ggzcore_net_send_perm_admin(net, player, perm, set) < 0) {
+		return -1;
+	}
+
+#if 0
+	/* Setting this automatically would improve situation with lag but
+	   presents bigger problems if the operation fails at the server
+	   end. */
+	_ggzcore_player_set_perm(&player->perms, perm, set);
+#endif
+
+	return 0;
+}
+
+
 int _ggzcore_player_get_lag(const GGZPlayer * player)
 {
 	return player->lag;
@@ -322,13 +382,18 @@ int _ggzcore_player_get_highscore(const GGZPlayer * player, int *highscore)
 
 int _ggzcore_player_compare(const void *p, const void *q)
 {
-	const GGZPlayer *s_p = p;
-	const GGZPlayer *s_q = q;
+	const GGZPlayer *s_p = p, *s_q = q;
 
-	return strcmp(s_p->name, s_q->name);
+	if (s_p->name && s_q->name) {
+		return strcmp(s_p->name, s_q->name);
+	} else {
+		/* This shouldn't happen, but it has been known to. */
+		return (s_p->name ? 1 : 0) - (s_q->name ? 1 : 0);
+	}
 }
 
 
+#if 0 /* deprecated */
 void *_ggzcore_player_create(void *p)
 {
 	GGZPlayer *new, *src = p;
@@ -339,6 +404,7 @@ void *_ggzcore_player_create(void *p)
 
 	return (void *)new;
 }
+#endif
 
 
 void _ggzcore_player_destroy(void *p)
@@ -365,6 +431,24 @@ void _ggzcore_room_set_player_lag(GGZRoom * room, const char *name,
 }
 
 
+void _ggzcore_room_set_player_perms(GGZRoom * room, const char *name,
+				    GGZPermset perms, GGZPlayerType type)
+{
+	/* FIXME: This should be sending a player "class-based" event */
+	GGZPlayer *player;
+
+	ggz_debug(GGZCORE_DBG_ROOM, "Setting lag to 0x%08X for %s",
+		  perms, name);
+
+	player = _ggzcore_room_get_player_by_name(room, name);
+	/* make sure they're still in room */
+	if (player && player->perms != perms) {
+		player->perms = perms;
+		player->type = type;
+		_ggzcore_room_event(room, GGZ_PLAYER_PERMS, name);
+	}
+}
+
 void _ggzcore_room_set_player_stats(GGZRoom * room, GGZPlayer * pdata)
 {
 	/* FIXME: This should be sending a player "class-based" event */
