@@ -4,7 +4,7 @@
  * Project: ggzmod
  * Date: 10/14/01
  * Desc: GGZ game module functions, GGZ side
- * $Id: ggzmod-ggz.c 8810 2007-01-04 21:11:55Z jdorje $
+ * $Id: ggzmod-ggz.c 9709 2008-02-13 11:23:40Z oojah $
  *
  * This file contains the backend for the ggzmod library.  This
  * library facilitates the communication between the GGZ core client (ggz)
@@ -417,9 +417,11 @@ static void _ggzmod_ggz_set_seat(GGZMod *ggzmod, GGZSeat *seat)
 
 static GGZSeat _ggzmod_ggz_get_seat(GGZMod *ggzmod, int num)
 {
-	GGZSeat seat = {num: num,
-			type: GGZ_SEAT_NONE,
-			name: NULL};
+	GGZSeat seat = {
+		.num = num,
+		.type = GGZ_SEAT_NONE,
+		.name = NULL
+	};
 
 	if (num >= 0 && num < ggzmod->num_seats) {
 		GGZListEntry *entry;
@@ -479,7 +481,10 @@ static void _ggzmod_ggz_set_spectator_seat(GGZMod * ggzmod, GGZSpectatorSeat *se
 
 static GGZSpectatorSeat _ggzmod_ggz_get_spectator_seat(GGZMod * ggzmod, int num)
 {
-	GGZSpectatorSeat seat = {num: num, name: NULL};
+	GGZSpectatorSeat seat = {
+		.num = num,
+		.name = NULL
+	};
 
 	if (num >= 0 && num < ggzmod->num_spectator_seats) {
 		GGZListEntry *entry;
